/*
 * @(#)cdemoextapi.c    1.00   10/29/04
 *
 * Copyright (c) 2004-Present Agilis Software LLC, Santa Clara, CA, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Agilis Software LLC ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only
 * in accordance with the terms of the license agreement you entered
 * into with Agilis Software LLC.
 *
 * DISCLAIMER: this sample code is provided as-is.
 */

/*
 * This demo illustrates how to securely manage state in an
 * EasyLicenser-enabled application using the extended
 * API's for maintaining quota consumption, deferred-time-limited
 * licensing, deferred node locking, and other license-related 
 * application state.
 *
 * The demo is invoked in one of 3 modes:
 * Activation: this is equivalent to an installation or activation of
 * a protected application.
 * Execution: this is equivalent to an execution of a protected application
 * in the steady state.
 * Deactivation: this is equivalent to an uninstallation or explicit
 * deactivation of a protected application.
 * Running the demo once in execution mode is equivalent to your end 
 * customer running your license-protected application at their site 
 * and consuming a specified amount of quota at their site.  
 * The quota consumption is specified as a command line argument.
 *
 * Features illustrated with this demo:
 * 1. How to use the extended EasyLicenser API calls in conjunction with
 *    the key cookie mechanism to manage and update license state 
 *    including the built-in quota management and the extensibility 
 *    mechanism to manage application-specific state, as well as to
 *    illustrate deferred-time-limited licensing and deferred node
 *    locking.
 * 2. How to utilize the application-password security mechanism.
 * 3. How to protect against system clock changes without writing code.
 *
 * For examples on using custom key handlers for extensibility,
 * as well as using digital signatures and other anti-spoofing measures,
 * see the accompanying "cdemosecure.cpp" demo application.
 * For an example of using the core API to accomplish secure state
 * management, see the accompanying "cdemostate.c" demo application.
 *
 * Some important points to note about this demo:
 * 1. The code that is directly relevant to using the EasyLicenser API's
 *    is indicated as such in comments.  The rest of the code exists in
 *    order to have a functioning self-contained program.
 * 2. All code pertaining to reading the key from an EasyLicenser file is
 *    for functional purposes only.  It is not meant to suggest a specific
 *    technique for managing the key in your application.  Nor is it
 *    a part of the EasyLicenser product.  In particular, note that your
 *    actual application will typically have access to just the license key
 *    without any accompanying metadata about the key.
 * 3. The demo will work with keys generated with EasyLicenser 2.0 as well
 *    as prior versions.  The EasyLicenser 2.0 run time library is required.
 *    *** Important note: unlike the other demo applications (C_Demo,
 *    cdemostate and cdemosecure), only the ".key" format is accepted.
 * 4. This demo performs automatic state management in a file that persists
 *    past its execution.  Therefore, each successive run will have a 
 *    different outcome and eventually you should encounter an "exceeded
 *    quota" error if you supplied a quota limited key, or a "license
 *    expired" error if you specified an auxiliary trial-generator key
 *    (even if your key was not time limited).
 * 5. Unlike the basic "C-demo.c", this demo processes only the first key
 *    in its input file.  Therefore, provide a file containing only one key.
 *    The key need not be quota limited.  If it isn't, the key cookie will
 *    still manage a use count, which can be useful in an actual application.
 * 6. The command line arguments to the demo are different in style from the
 *    other demos (C_Demo, cdemostate and cdemosecure), and the input license
 *    key is inputted as a file input parameter which is also used to input
 *    the key cookie.  Run the demo without arguments for complete help
 *    information.
 *
 */


#define __STRICT_ANSI__ 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include <ezLicenseInfo.h>
#include <ezLicenseExtAPI.h>

#define KEY_MAXLENGTH 8000
#define MAX_PROD_LENGTH 200
#define MAX_HFILE_DIR_LENGTH 200

char product[MAX_PROD_LENGTH];
char appPwd [ MAX_APP_PWD_LENGTH ];
long quotaIncr;
char command [100];
char userhost [ MAX_USER_HOST_NET_NAME_LENGTH ];
char *userhostp = &userhost[0];
char auxkeyfilename [ KEY_MAXLENGTH ];
char lickeyfilename[KEY_MAXLENGTH];
char hfileDir [MAX_HFILE_DIR_LENGTH];
char stateBuf[MAX_KEY_COOKIE_SIZE];

char ezlmMajorVersion[MAX_VERSION_INFO_LENGTH];
char ezlmMinorVersion[MAX_VERSION_INFO_LENGTH];

int strip(char *instr, char retbuf[], size_t retbufsize);
void displayLicenseParams(void* ezLicInstance);
void showLicenseModel(int licenseModel,int licenseType);
void checkWarnings(int warnings);
void makeDateStr(time_t ltime, char dateStrBuf[], size_t dateStrBufsize);
void displayExtendedLicenseState(void* ezLicExtInstance);
void displayError(int errorCode);


static void init() {
	product[0] = '\0';
	appPwd[0] = '\0';
	quotaIncr = 0;
	command[0] = '\0';
	userhost[0] = '\0';
	auxkeyfilename[0] = '\0';
	lickeyfilename[0] = '\0';
	hfileDir[0] = '\0';
}

void reportParams() {
    fprintf(stderr, "\nRuntime library version information:\n");
    if (getEzlmVersionInfoStr(ezlmMajorVersion, sizeof(ezlmMajorVersion),
                              ezlmMinorVersion, sizeof(ezlmMinorVersion)) < 0) {
        fprintf(stderr, "*** Error getting EasyLicenser runtime version information ***\n");
        exit (2);
    }
    fprintf(stderr,"  Major Version: %s, Minor Version: %s\n", ezlmMajorVersion, ezlmMinorVersion);

	fprintf(stderr,"Input / Default Parameters:\n");
	fprintf(stderr,"  Command : %s\n", command);
	fprintf(stderr,"  Product name:     %s\n", product);
	fprintf(stderr,"  Application password public key: %s\n", (appPwd[0] == '\0' ? "<<not specified>>":appPwd));
	fprintf(stderr,"  License key/key cookie filename :     %s\n", lickeyfilename);
	fprintf(stderr,"  Logical user / host name in license key: %s\n", (userhost[0] == '\0' ? "<<not specified>>": userhost));
	fprintf(stderr,"  Auxiliary Agilis license key input file: %s\n", (auxkeyfilename[0] == '\0' ? "<<not specified>>": auxkeyfilename));
	if(quotaIncr <= 0)
		fprintf(stderr,"  Metered-license quota consumption increment:   %s\n", "<<not specified>>");
	else
		fprintf(stderr,"  Metered-license quota consumption increment:   %ld\n", quotaIncr);
	fprintf(stderr,"  Hidden file directory: %s\n",(hfileDir[0] == '\0'? "<<not specified>>": hfileDir));
}


void printUsage(const char *progname) {
		fprintf(stderr,"Usage: cdemoextapi -cCommand -fLicenseKeyfile -pProduct [-wPassword] [-uUserhost] [-aAuxLicensefile] [-qQuotaIncrement] [-rHiddenFileDir]\n");
		fprintf(stderr,"  -cCommand: one of 'activate', 'check' or 'deactivate'.  Required.\n");
		fprintf(stderr,"  -fLicenseKeyfile: Name of file containing raw license key ('activate') / key cookie.  Required.\n");
		fprintf(stderr,"  -pProduct: name of product for which license key / key cookie is defined.  Required.\n");
		fprintf(stderr,"  -wPassword: application password public key.  Optional, defaults to null.\n");
		fprintf(stderr,"  -uUserhost: logical user/host name matching license key.  Optional, defaults to pre-node-locked.\n");
		fprintf(stderr,"  -aAuxLicenseFile: name of file containing auxiliary Agilis license key for the product.  Optional.\n");
		fprintf(stderr,"  -qQuotaIncrement: amount by which to increment quota consumption.  Optional, defaults to 1.\n");
		fprintf(stderr,"  -rHiddenFileDir: hidden file directory.  Optional, defaults to current working directory.\n");
		fprintf(stderr,"Example:\n");
		fprintf(stderr,"   cdemoextapi -pTestDemo -w\"5Bx4W4=\" -udummyuser -cactivate -fkeyTestDemo.key -aAgilisEnablerKey.key > cookie1.txt\n");
		fprintf(stderr,"   cdemoextapi -pTestDemo -w\"5Bx4W4=\" -udummyuser -ccheck -fcookie1.txt -aAgilisEnablerKey.key > cookie2.txt\n");
		fprintf(stderr,"   cdemoextapi -pTestDemo -w\"5Bx4W4=\" -udummyuser -cdeactivate -fcookie2.txt -aAgilisEnablerKey.key\n");

		exit(2);
}


void getArgs(int argc, char *argv[]) {
	int i;
	char* arg = NULL ;

	if (argc <= 1)
		printUsage(argv[0]);
	
	for (i = 1; i < argc; i++) {
		arg = argv[i];
		if (!strncmp(arg, "-p", 2)) {
			if (strlen(arg) == 2) {
				fprintf(stderr,"*** No argument specified for -p ***\n");
				exit(1);
			}
			strcpy(product, &arg[2]);
		} else if (!strncmp(arg, "-c", 2)) {
			if (strlen(arg) == 2) {
				fprintf(stderr,"*** No argument specified for -c ***\n");
				exit(1);
			}
			strcpy(command, &arg[2]);
			if (strcmp(command, "activate") && strcmp(command, "check") &&
			    strcmp(command, "deactivate")) {
				fprintf(stderr, "*** Invalid command - must be activate / check / deactivate.\n");
				exit(1);
			}
		} else if (!strncmp(arg, "-w", 2)) {
			if (strlen(arg) == 2) {
				fprintf(stderr,"*** No argument specified for -w ***\n");
				exit(1);
			}
			strcpy(appPwd, &arg[2]);
		} else if (!strncmp(arg,"-u",2)) {
			if (strlen(arg) == 2) {
				fprintf(stderr,"*** No argument specified for -u ***\n");
				exit(1);
			}
			if (!strcmp(&arg[2],"null"))
				userhostp = NULL;
			else
				strcpy(userhost, &arg[2]);
		} else if (!strncmp(arg,"-f",2)) {
			if (strlen(arg) == 2) {
				fprintf(stderr,"*** No argument specified for -f ***\n");
				exit(1);
			}
			strcpy(lickeyfilename, &arg[2]);
		}else if (!strncmp(arg,"-a",2)) {
			if (strlen(arg) == 2) {
				fprintf(stderr,"*** No argument specified for -a ***\n");
				exit(1);
			}
			strcpy(auxkeyfilename, &arg[2]);
		} else if (!strncmp(arg,"-q",2)) {
			if (strlen(arg) == 2) {
				fprintf(stderr,"*** No argument specified for -q ***\n");
				exit(1);
			}

			if ((quotaIncr = atol(&arg[2])) <= 0) {
				fprintf(stderr, "*** Numeric value for -q must be a positive integer ***\n");
				exit(1);
			}
		} else if (!strncmp(arg,"-r",2)) {
			if (strlen(arg) == 2) {
				fprintf(stderr,"*** No argument specified for -r ***\n");
				exit(1);
			}
			strcpy(hfileDir, &arg[2]);
		} else {
			if (strcmp(arg, "-h"))
				fprintf(stderr,"*** Invalid argument: %s ***", arg);
			printUsage(argv[0]);
		}
	}
	if (!(product[0] && command[0] && lickeyfilename[0])) {
		printUsage(argv[0]);
		exit(1);
	}
}


char * readKey ( char * fileName)
{
	int i;
	char licKey [MAX_KEY_COOKIE_SIZE];
	char * keyBuf = NULL;
	keyBuf = (char *) malloc(MAX_KEY_COOKIE_SIZE);

	// read from standard input
	if(fileName == NULL)
	{
		fgets(licKey, sizeof(licKey), stdin);
	}
	else
	{
		FILE *fh = fopen(fileName,"r");
		if (!fh) {
			fprintf(stderr,"\n*** Cannot open license key file '%s' ***\n",	fileName);
			return keyBuf;
		}
	
		// read key from file
		fgets(licKey, sizeof(licKey), fh);

		fclose(fh);
	}

	// strip the key
	//printf("in getkey = %s , size = %d",licKey , strlen(licKey));
	licKey[strlen(licKey)] = '\0';
	i = strip(licKey, keyBuf, MAX_KEY_COOKIE_SIZE);
	//printf("in getkey = %s ,  i = %d",keyBuf , i);
	return keyBuf;
}

/* 
 * Strip the key of all leading, trailing and embedded white spaces.
 * A good idea for an actual application, too.
 */
int strip(char *instr, char retbuf[], size_t retbufsize)
{
	char *outptr = &retbuf[0];
	int len = 0;
	char ch = 0;
	if (retbufsize <= strlen(instr))
		return -1;
	while (ch = *instr++) {
		if (!isspace((int)ch))
			*outptr++ = ch;
	}
	*outptr = '\0';
	return 0;
}

void doActivate(char* keytxt, char* auxkeytxt) {
		int status ;
		char strCookie[10240];
		boolean deferredNodeLock;
		void * licInfo = 0;
		licInfo = (void *) malloc(sizeof(long));
		
		fprintf(stderr,"Activating license...");
		
		if(auxkeytxt != NULL)
			deferredNodeLock = TRUE;
		else
			deferredNodeLock = FALSE;

		// EasyLicenser Extended API call: activate license, possibly performing
		// a deferred node lock and initializing deferred time limited licensing
		// if an appropriate auxiliary Agilis license key is provided.
		status = activateExtAPILicense(
				keytxt, 		// actual license key text
				auxkeytxt,		// auxiliary Agilis license key text
				0,			// no custom key handler in this example
				0,			// no custom key context in this example
				userhostp,		// logical user/host name (does not have to be
								// OS-level user/host if using deferred node
								// locking.
				product,		// case-sensitive product name
				appPwd,			// product application password public key
				"userappstateexample", // user-specified application state
				hfileDir,	// hidden file directory
				TRUE,			// check system clock
				deferredNodeLock, // deferred node locking only if 
								// auxiliary key specified
				&licInfo
				);
	
		if(status < 0)
		{
			fprintf(stderr,"\n Error Activating License.");
			displayError(status);
			exit(2);
		}
		fprintf(stderr,"\nActivated license.\n");
		checkWarnings(getLicenseExtAPIStatusCode(licInfo));
		displayLicenseParams(getLicenseExtAPILicenseInfo(licInfo));
		displayExtendedLicenseState(licInfo);
		status = getLicenseExtAPIKeyCookieStr(licInfo,strCookie,sizeof(strCookie));
		printf("%s",strCookie);
}


void doCheck(char* keytxt) {
		int status ;
		char strCookie[10240];
		void * licInfo = 0;
		licInfo = (void *) malloc(sizeof(long));

		fprintf(stderr,"Check license...");
		// EasyLicenser Extended API call: check license, possibly performing
		// checks for deferred node locking and deferred time limited licensing
		// if an appropriate auxiliary Agilis license key was provided at
		// the time of activation.
		//printf("\nKey text = %s" , keytxt);
		status = checkExtAPILicense(
				keytxt, 		// actual license key cookie text
				0,			// no custom key handler in this example
				0,			// no custom key context in this example
				5,				// days-left warning threshold
				-quotaIncr,		// quota usage to date: -ve value implies
								// quota to deduct as for secure API calls
				10,				// quota warning threshold
				0,				// past-expiration days grace
				0,				// over-quota grace
				userhostp,		// logical user/host name (does not have to be
								// OS-level user/host if using deferred node
								// locking.
				0,				// current usage level - not interesting for
								// this demo
				product,		// case-sensitive product name
				appPwd,			// product application password public key
				NULL, 			// user-specified application state unchanged
				hfileDir,	// hidden file directory
				TRUE,			// check system clock
				&licInfo
			);
		
		if(status < 0)
		{
			fprintf(stderr,"\n Error Checking License.");
			displayError(status);
			exit(2);
		}
		fprintf(stderr,"Checked license.");
		checkWarnings(getLicenseExtAPIStatusCode(licInfo));
		displayLicenseParams(getLicenseExtAPILicenseInfo(licInfo));
		displayExtendedLicenseState(licInfo);
		status = getLicenseExtAPIKeyCookieStr(licInfo,strCookie,sizeof(strCookie));
		printf("%s",strCookie);
	}

 void doDeactivate(char* keytxt) {
		int status;
		fprintf(stderr,"\nDeactivating license...");
		// EasyLicenser Extended API call: deactivate license, after
		//first performing a lenient check on it to ensure it is
		// a valid extended-API key cookie for the product.
		status = deactivateExtAPILicense(keytxt, product, appPwd, hfileDir, TRUE);
		if(status <0 )
			displayError(status);
		else
			fprintf(stderr,"\n License deactivated."); 
	}


	/**
	 * main entry point:
	 * Accepts mandatory command line arguments:
	 * 1. Case-sensitive product name that was specified at the time of key creation.
	 *    This is required even if the product does not have an associated 
	 *    application password.
	 * 2. Command: activate / check / deactivate
	 * and the optional arguments
	 * 3. Name of a text file containing a license key to be activated
	 *    if invoked in activation mode, or containing a key cookie in
	 *    .key notation if invoked in license check or deactivation mode.
	 *    If not specified, the license or key cookie is read from
	 *    the standard input stream, in which case it is expected to
	 *    be just the key with no additional formatting information.
 	 * 4. Quota increment: amount by which to increment quota consumption.
 	 *    To reset the saved state, specify a value of 0 to simulate
 	 *    product install time.
	 * 5. Expected user / host name that should match what's in the key.
     *    If the key is known to have been generated with the "enforce OS
     *    user / host" flag, you don't need to specify this parameter.
     *    Also, if invoked in deferred node locking mode, this name does 
     *    not have to be the OS user/host name.
	 * 6. Product's associated password's case-sensitive public key, available with
	 *    the product definition and also obtainable by invoking PkCrypt.makePublicKey(password)
	 *    and with the "makepubkey" Java utility located at ezlic2.0/util.
	 * 7. Auxiliary Agilis key, required if deferred time-limited licensing and deferred
	 *    node locking functionality are to be used.  This is a special key provided to the vendor
	 *    by Agilis Software.  It cannot be generated by the vendor.
	 * When invoked in activation or license check modes, the newly-generated key cookie is
	 * outputted to the standard-out stream, which may be redirected to a disk file in the
	 * command line invocation.  All messages produced by the demo are outputted to standard
	 * error.
	 */

int main(int argc, char *argv[]) {
	int ret = FALSE;
	char* keytxt ;
	char* auxkeytxt;
	char strBuffer[40];
    time_t nowTs = 0;
	time(&nowTs);

	init();
	getArgs(argc, argv);

	fprintf(stderr,"EasyLicenser Extended API Demo.");
	makeDateStr(nowTs, strBuffer, sizeof(strBuffer));
	fprintf(stderr,"    \nRunning On:%s\n ",strBuffer);

	reportParams();

	// set default value 
	if (quotaIncr <= 0)
		quotaIncr = 1;

	keytxt = readKey(lickeyfilename); // read key/state from the file specified
	auxkeytxt = auxkeyfilename[0] == '\0' ? NULL : readKey(auxkeyfilename);

	if (strcmp(command, "activate")==0)
		doActivate(keytxt, auxkeytxt);
	else if (strcmp(command, "check")==0)
		doCheck(keytxt);
	else if (strcmp(command, "deactivate")==0)
		doDeactivate(keytxt);
	else {
		fprintf(stderr,"*** Unrecognized command: %s ***" ,command );
		exit(2);
	}
	
	fprintf(stderr,"\nEnd demo.\n");
	return 0;
}

void checkWarnings(int warnings)
	{
		if (warnings != 0) {
			fprintf(stderr,"\n    Warnings: %d" , warnings);
			if ((warnings & EZLIC_WARN_EXPIRING) != 0)
				fprintf(stderr,"        Time Limited Key Is About To Expire");
			else if ((warnings & EZLIC_WARN_QUOTA) != 0)
				fprintf(stderr,"        Quota Limited Key Is About To Expire");
			else if ((warnings & EZLIC_WARN_EXPIRED) != 0)
				fprintf(stderr,"        Time Limited Key Has Expired");
			else if ((warnings & EZLIC_WARN_QUOTA_EXCEEDED) != 0)
				fprintf(stderr,"        Quota Limited Key Has Expired");
			else if ((warnings & EZLIC_WARN_EXP_GRACE_TRUNC) != 0)
				fprintf(stderr,"        Expiration Days Grace In API Call Too Large - Truncated To 5 / 25% Of Lifespan");
			else if ((warnings & EZLIC_WARN_QUOTA_GRACE_TRUNC) != 0)
				fprintf(stderr,"        Quota Grace In API Call Too Large - Truncated To 5 / 25% Of Original Quota");
			fprintf(stderr,"");
		}
	}

/*
*	Display the meaning of the -ve error code returned from the
*   license key API check.  Error codes are defined in <ezLicStatusCodes.h>
*/
void displayError(int errorCode)
{
	if (!errorCode)
		return;

	printf("*** ");
	switch(errorCode) {
	case     EZLIC_EXC_BASE:
			fprintf(stderr,"Error: General license validation failure. Please check your license key.\n");
			break;
	case     EZLIC_EXC_MALFORMED_KEY:
			fprintf(stderr,"Error: the license key is invalid.\n");
			break;
	case     EZLIC_EXC_EXPIRED:
			fprintf(stderr,"Error: The license has expired past the grace period.\n");
			break;
	case     EZLIC_EXC_EXCEEDED_LIMITS:
			fprintf(stderr,"Error: The licensed user / server / CPU count / quota limit in the key has been exceeded.\n");
			break;
	case     EZLIC_EXC_INVALID_HOST_USER_NET:
			fprintf(stderr,"Error: The user / machine name in the license key does not match your environment / parameter.\n");
			break;
	case     EZLIC_EXC_INVALID_CUSTOM_KEY:
			fprintf(stderr,"Error: The custom key contained in the license key is invalid.\n");
			break;
	case     EZLIC_EXC_UNIMPLEMENTED:
			fprintf(stderr,"Error: The feature you are using is not implemented.\n");
			break;
	case     EZLIC_EXC_GEN:
			fprintf(stderr,"Error: Key generation execption (you shouldn't encounter this condition).\n");
			break;
	case     EZLIC_EXC_VENDOR:
			fprintf(stderr,"Error: Vendor license key exception (you shouldn't encounter this condition).\n");
			break;
	case     EZLIC_EXC_INTERNAL:
			fprintf(stderr,"Error: internal error in EasyLicenser - please submit a bug report.\n");
			break;
	case     EZLIC_EXC_OS:
			fprintf(stderr,"Error: An operating system error occurred - please check your machine configuration.\n");
			break;
	case     EZLIC_EXC_CONFIG:
			fprintf(stderr,"Error: exception in license configuration - please check your EasyLicenser installation, system clock and (for deferred node-locked licenses) license key shelf life.\n");
			break;
	case     EZLIC_EXC_UNLICENSED_OPTION:
			fprintf(stderr,"Error: you attempted to perform an unlicensed operation.\n");
			break;
	case     EZLIC_EXC_USAGE:
			fprintf(stderr,"Error: API usage - please check your API call parameters and call sequence.\n");
			break;
	case     EZLIC_EXC_AUTHENT:
			fprintf(stderr,"Error: Key / cookie authentication failure - please check your product name and application password.\n");
			break;
	case     EZLIC_EXC_LIC_ALREADY_ACTIVE:
			fprintf(stderr,"Error: License key is already active. Deactivate it first.\n");
			break;
	case     EZLIC_EXC_LIC_INACTIVE:
			fprintf(stderr,"Error: License key is not active. Activate it first.\n");
			break;
	case     EZLIC_EXC_PERPETUAL_USE:
			fprintf(stderr,"Error: License key is not for perpetual use.\n");
			break;
	case     EZLIC_EXC_INVALID_AUXKEY:
			fprintf(stderr,"Error: Agilis license enabler key is either not for use with the specified product or is otherwise invalid.\n");
			break;
	case     EZLIC_EXC_TAMPERED_CLOCK:
			fprintf(stderr,"Error: Tampered system clock.\n");
			break;
	case     EZLIC_EXC_INSTALLED_LATE:
			fprintf(stderr,"Error: Key installed too late.\n");
			break;
	default:
			fprintf(stderr,"Error: unknown error code %d.\n", errorCode);
			break;
	}
}


/*
* Introspect the license key and display its parameters.
* In an actual application you may introspect specific
* values and act on them instead of displaying them.
* The most common parameter that is post-processed is
* the product options.
*/
void  displayLicenseParams(void* ezLicInstance)
{
	int value = 0;
	long lvalue = 0L;
	long ltime = 0;
	char* strValue = 0;


	fprintf(stderr,"\n    License Key Information:\n");
	fprintf(stderr,"        Key version:    ");
	fprintf(stderr,(getKeyVersion(ezLicInstance) <= 0 ? "Legacy (1.1 or earlier)\n":"%d\n"),
		   getKeyVersion(ezLicInstance));
	strValue = getChainedKeyId(ezLicInstance);
	if (strValue && strValue[0])
		fprintf(stderr,"        Replacement for key with Id: '%s'\n", strValue);

	value = getLicenseModelCode(ezLicInstance);
	showLicenseModel(value, getLicenseTypeCode(ezLicInstance));
	if (getEnforce(ezLicInstance))
		fprintf(stderr,"        Operating System check enforced.\n");

	value = getLicenseModeBitmap(ezLicInstance);
	fprintf(stderr,"        License Mode Bitmap:    0x%x\n", value);
	if (value & EZLIC_MODE_TIME)
		fprintf(stderr,"          TIME LIMITED\n");
	if (value & EZLIC_MODE_OPTIONS)
		fprintf(stderr,"          OPTIONS\n");
	if (value & EZLIC_MODE_METERED)
		fprintf(stderr,"          METERED\n");
	if (value & EZLIC_MODE_CUSTOM_KEY)
		fprintf(stderr,"          CUSTOM KEY\n");
	if (value & EZLIC_MODE_CUSTOM_COOKIE)
		fprintf(stderr,"          CUSTOM COOKIE\n");

	lvalue = getUsageValue(ezLicInstance);
	if (lvalue > 0)
		fprintf(stderr,"        Usage Limit Value:    %ld\n", lvalue);

	lvalue = getQuotaValue(ezLicInstance);
	if (lvalue > 0)
		fprintf(stderr,"        Quota Limited To:    %ld\n", lvalue);

	strValue = getOptions(ezLicInstance);
	if (strValue  && strValue[0])
		fprintf(stderr,"        Options:    [%s]\n", strValue);

	if ((ltime = getExpirationTime(ezLicInstance)) > 0) {
		char tmpbuf[100];
		makeDateStr(ltime, tmpbuf, sizeof(tmpbuf));
		fprintf( stderr,"        Time Limited To Expire (GMT):    %s\n", 
				tmpbuf);
		
	}
   	
	strValue = getUserHostNetName(ezLicInstance);
	if (strValue && strValue[0])
		fprintf(stderr,"        User/Host Name:    [%s]\n", strValue);
	strValue = getCustomCookie(ezLicInstance);
	if (strValue && strValue[0])
		fprintf(stderr,"        Custom Cookie:    [%s]\n", strValue);
	strValue = getCustomKey(ezLicInstance);
	if (strValue && strValue[0])
		fprintf(stderr,"        Custom Key:    [%s]\n", strValue);
	strValue = getAppPwd(ezLicInstance);
	if (strValue && strValue[0])
		fprintf(stderr,"        Application Password:    [%s]\n", strValue);
}

void  showLicenseModel(int licenseModel,
										 int licenseType)
{
	fprintf(stderr,"        License Model: %s\n", 
			(licenseModel == EZLIC_MODEL_USER ? "User":
			 licenseModel == EZLIC_MODEL_SERVER ? "Server":
			 "*** Unknown ***"));
	fprintf(stderr,"        License Type: ");
	switch(licenseModel) {
	case EZLIC_MODEL_USER:
		switch(licenseType) {
		case EZLIC_TYPE_U_USER:
			fprintf(stderr,"Named User\n");
			break;
		case EZLIC_TYPE_U_NODE:
			fprintf(stderr,"Node Locked\n");
			break;
		case EZLIC_TYPE_U_FLOAT:
			fprintf(stderr,"Floating User\n");
			break;
		default:
			fprintf(stderr,"*** Unknown license type %d ***\n", licenseType);
		}
		break;
	case EZLIC_MODEL_SERVER:
		switch(licenseType) {
		case EZLIC_TYPE_SVR_CONC:
			fprintf(stderr,"Concurrent User\n");
			break;
		case EZLIC_TYPE_SVR_CPU:
			fprintf(stderr,"CPU Count\n");
			break;
		case EZLIC_TYPE_SVR_MHZ:
			fprintf(stderr,"CPU MHZ\n");
			break;
		case EZLIC_TYPE_SVR_NMU:
			fprintf(stderr,"Named User\n");
			break;
		default:
			fprintf(stderr,"*** Unknown license type %d ***\n", licenseType);
		}
		break;
	default:
		fprintf(stderr,"%d\n", licenseModel);
	}
}

/*
	 * Display state contents of extended-license contained in key cookie.
	 * Note that you can introspect a key cookie but you can't create /
	 * update it outside the EasyLicenser extended API.  In an actual
	 * application, the only item that may be interesting to the
	 * application would be the application-specified state information
	 * provided at the time of an activation or license check.
	 */
void displayExtendedLicenseState(void* ezLicExtInstance) {
		char temp[20];
		char strValue[1000];
		char strCookie[10240];
		char strBuffer[MAX_APP_STATE_SIZE];
		int status;
		long ltime = 0;
		time_t updateTime = 0;
		long quotaToDate = 0 ;

		fprintf(stderr,"\nExtended License State:");

		status = getLicenseExtAPIKeyCookieStr(ezLicExtInstance,strCookie,sizeof(strCookie));
		fprintf(stderr,"\n(a) Transient state from last operation:");
		fprintf(stderr,"\n    Key cookie: %s " , strCookie);
		fprintf(stderr,"\n    Operation status code: %d" , getLicenseExtAPIStatusCode(ezLicExtInstance));
	


		fprintf(stderr,"\n(b) Persistent state:");
		status = getLicenseExtAPILicenseKeyStr(ezLicExtInstance,strValue,sizeof(strValue));
		strncpy(temp,strValue,19);
        temp[19]='\0';
	    fprintf(stderr,"\n    Original license key (First few characters) :%s ...",temp);

	
		status = getLicenseExtAPIUserAppstateStr(ezLicExtInstance,strBuffer,sizeof(strBuffer));
		fprintf(stderr,"\n    User application state: %s\n", strBuffer );

		status = getLicenseExtAPIUserhostStr(ezLicExtInstance,strValue,sizeof(strValue));
		fprintf(stderr,"    OS User/host name: %s\n" , strValue);

		if ((ltime = getLicenseExtAPIActivationTs(ezLicExtInstance)) > 0) {
			makeDateStr(ltime, strBuffer, sizeof(strBuffer));
			fprintf(stderr, "    Activation date/time (GMT): %s\n", strBuffer);
		}

		if ((ltime =getLicenseExtAPIExpirationTs(ezLicExtInstance)) > 0){
			makeDateStr(ltime, strBuffer, sizeof(strBuffer));
			fprintf(stderr,"    Expiration date/time:%s\n ",strBuffer);
		}else
			fprintf(stderr,"     Not a deferred-time-limited license.\n");

		fprintf(stderr,"\n(c) Key cookie internal state:");

		// EasyLicenser API call: get quota-consumed-to-date state from key cookie 

		if (getKeyCookieQuotaToDate(strCookie, product, appPwd,
					   &quotaToDate) < 0) {
			fprintf(stderr,"\n*** Error getting quota to date ***\n");
		} 
		  
		fprintf(stderr," \n   Quota-to-date: %ld" , quotaToDate );

		/* EasyLicenser API call: get cookie update timestamp from key cookie */
		if (getKeyCookieTs(strCookie, product, appPwd,
					   &updateTime) < 0) {
			fprintf(stderr,"*** Error getting update timestamp ***\n");
		} else {
			makeDateStr(updateTime, strBuffer, sizeof(strBuffer));
			fprintf(stderr," \n   Update date/time (GMT): %s (timestamp: %ld)\n", strBuffer, updateTime);
		}
}

void makeDateStr(time_t ltime, char dateStrBuf[], size_t dateStrBufsize)
{
	char endch;
	struct tm when;
	if (ltime == 0)
		time(&ltime);
	when = *gmtime(&ltime);
	strncpy(dateStrBuf, asctime(&when), dateStrBufsize-1);
	endch = dateStrBuf[strlen(dateStrBuf)-1];
	if (iscntrl((int)endch))
		dateStrBuf[strlen(dateStrBuf)-1]='\0';
}
