/*
 * @(#)MacAddrChecker.java    1.00   11/14/03
 *
 * Copyright (c) 2003 Agilis Software LLC, Fremont, CA, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Agilis Software LLC ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only
 * in accordance with the terms of the license agreement you entered
 * into with Agilis Software LLC.
 */

import java.text.*;
import java.util.*;
import java.io.*;
import java.net.*;

// import com.vs.ezlicrun.*;

/*
 * This is a code fragment that illustrates how to plug in Ethernet MAC
 * address checks into an EasyLicenser-enabled Java application for 
 * multiple platforms. The code fragment is incomplete
 * code intended to be used as a head start, and will not compile by
 * itself.
 *
 * The implementation includes working code for Windows, Linux, Solaris,
 * HPUX, AIX and SGI-IRIX, and an infrastructure for extending support 
 * to additional platforms. The implementation scans all available 
 * MAC addresses for a value that matches the input, which is expected 
 * to conform to the platform-specific syntax for a MAC address.
 * The infrastructure may need to be extended to accommodate platforms
 * whose command line utility output does not conform to the
 * structure that is assumed here.
 *
 * DISCLAIMER: this code is provided as-is.
 */

public class MacAddrChecker implements EzLicCustomLicenseInterface
{
	private static final int OS_WINDOWS = 1;
	private static final int OS_LINUX = 2;
	private static final int OS_HPUX = 3;
	private static final int OS_SOLARIS = 4;
	private static final int OS_AIX = 5;
	private static final int OS_IRIX = 6;
	private static final int OS_OTHER = 7;

	private static final String[] OsCmd = {null, "ipconfig /all", "/sbin/ifconfig -a", "lanscan", "/usr/sbin/arp -a", "netstat -v | grep Hardware", "nvram | grep eadd", "dunno"};
	private static final String[] MACkwd = {null, "Physical Address", "HWaddr", "N/A", "N/A", "Hardware Address", "eaddr", "dunno"};
	private static final String[] MACtoksep = {null, "-", ":", null, ":", ":", ":", "dunno"};
	private static final String[] MACsep = {null, ":", " ", "N/A", "N/A", ":", "=", "dunno"};

	private static int GetOsCode() throws EzLicExceptionBase {
		String sys = System.getProperty("os.name");
		if(sys.indexOf("Windows") >= 0)
			return OS_WINDOWS;
		if (sys.indexOf("Linux") >= 0)
			return OS_LINUX;
		if (sys.indexOf("Solaris") >= 0 || sys.toLowerCase().indexOf("sunos") >= 0)
			return OS_SOLARIS;
		if (sys.toLowerCase().indexOf("hp") >= 0)
			return OS_HPUX;
		if (sys.toLowerCase().indexOf("aix") >= 0)
			return OS_AIX;
		if (sys.toLowerCase().indexOf("irix") >= 0)
			return OS_IRIX;
        EzLicExceptionBase.throwLicenseException(
            EzLicExceptionBase.EZLIC_EXC_CONFIG, 
            	"Unsupported OS for MAC Address Checking: " + sys);
		return 0;
	}

	public int checkCustomKeyCode(String mcAddr, Object ctx) throws EzLicExceptionBase {
		boolean status = false;
		int os = GetOsCode();
		if (!checkIfMACAddr(mcAddr, MACtoksep[os]))
			EzLicExceptionBase.throwLicenseException(
					EzLicExceptionBase.EZLIC_EXC_MALFORMED_KEY, 
					"Unrecognized machine parameter");
		try {    
			if (!checkMachineAddress(mcAddr, os))
				EzLicExceptionBase.throwLicenseException(
					EzLicExceptionBase.EZLIC_EXC_INVALID_HOST_USER_NET , " Unlicensed Ethernet MAC Address");
      	} catch (Throwable t) {
			if (t instanceof EzLicExceptionBase)
				throw (EzLicExceptionBase)t;
        	EzLicExceptionBase.throwLicenseException(
            	EzLicExceptionBase.EZLIC_EXC_INTERNAL, 
            		"Unexpected Exception: " + t.getClass().getName() + ", Message: " + t.getMessage());
      	}
    	return 0;
  	}

  
	private static boolean checkMachineAddress(String inaddr, int os) throws Exception {
		Process p = Runtime.getRuntime().exec(OsCmd[os]);
		if (p == null) {
			throw new Exception("OS configuration error: can't run OS command '"+OsCmd[os]+"'to get machine address");
		}
        BufferedReader br = null;
        try {
            br = new BufferedReader(
            		new InputStreamReader(
            			p.getInputStream()));
            String sLine = null;
            while ((sLine = br.readLine()) != null) {
                sLine = sLine.trim();
				switch (os) {
				case OS_HPUX:
				{
					StringTokenizer st = new StringTokenizer(sLine, " \t");
					if (st.hasMoreTokens())
						st.nextToken();
					if (st.hasMoreTokens()) {
						String eth = st.nextToken();
						if (eth.startsWith("0x") && eth.equals(inaddr))
							return true;
					}
					break;
				}
				case OS_SOLARIS:
				{
					StringTokenizer st = new StringTokenizer(sLine, " \t");
					String eth = null;
					do {
						eth = st.nextToken();
					} while (st.hasMoreTokens()); 
					if (eth != null && checkIfMACAddr(eth, ":") && eth.equals(inaddr))
						return true;
					break;
				}
				case OS_AIX:
				{
					int idx = -1;
					while (sLine != null && (idx = sLine.indexOf(MACkwd[os])) >= 0) {
						if (idx > 0)
							sLine = sLine.substring(idx);
						sLine = sLine.substring(sLine.indexOf(MACsep[os])+1);
						if (sLine != null) {
							sLine = sLine.trim();
							if (sLine.startsWith(inaddr))
								return true;
						}
					}
					break;
				}
				default:
					int idx = -1;
					if ((idx = sLine.indexOf(MACkwd[os])) >= 0) {
						if (idx > 0)
							sLine = sLine.substring(idx);
						sLine = sLine.substring(sLine.indexOf(MACsep[os])+1);
						if (sLine != null) {
							sLine = sLine.trim();
							if (sLine.startsWith(inaddr))
								return true;
						}
					}
				}
            }
            return false;
        } finally {
            if (br != null) 
            	br.close();
        }
	}

	private static boolean checkIfMACAddr(String mcAddr, String toksep) {
		if (toksep == null) {
			if (!mcAddr.startsWith("0x"))
				return false;
			try {
				Long.parseLong(mcAddr.substring(2),16);
			} catch (Throwable t) {
				return false;
			}
			return true;
		}
		String tmp = mcAddr;
		int idx = -1;
		int i;
		for (i = 0, tmp=mcAddr; i < 5; i++, tmp = tmp.substring(idx+1)) {
			idx = tmp.indexOf(toksep);
			if (idx <= 0) {
				return false;
			}
		}
		return true;
	}

}
